@@ -1,6 +1,18 @@
 import mysql.connector, std, re, random, time, os
 
+def format_table(table):
+    if m := re.match('(\w+)\.(\S+)$', table):
+        return "%s.`%s`" % m.groups()
+    else:
+        return "`%s`" % table
+    
 class Database:
+    # usage:
+    '''vim ~/.bashrc
+export MYSQL_USER=yourName
+export MYSQL_PASSWORD=yourPassword
+export MYSQL_HOST=yourIPAddress
+    '''
 
     def create_database(self):
         cursor = self.cursor()
@@ -10,10 +22,11 @@ class Database:
             print("Failed creating database: {}".format(err))
 
     def __init__(self):
-        self.user = os.environ.get('MYSQL_USER', 'prod')
-        self.password = os.environ.get('MYSQL_PASSWORD', 'prod')
+        self.user = os.environ.get('MYSQL_USER', 'user')
+        self.password = os.environ.get('MYSQL_PASSWORD', 'user')
         self.host = os.environ.get('MYSQL_HOST', '127.0.0.1')
-        self.database = os.environ.get('MYSQL_DATABASE', 'axiom')
+        self.port = os.environ.get('MYSQL_PORT', '3306')
+        self.database = os.environ.get('MYSQL_DATABASE', 'corpus')
         self.charset = os.environ.get('MYSQL_CHARSET', 'utf8')
         try:
             self.conn = mysql.connector.connect(**self.kwargs)
@@ -33,7 +46,7 @@ class Database:
 
     @property
     def kwargs(self):
-        return dict(user=self.user, password=self.password, host=self.host, database=self.database, charset=self.charset)
+        return dict(user=self.user, password=self.password, host=self.host, database=self.database, charset=self.charset, port=self.port)
 
     def cursor(self, **kwargs):
         return self.conn.cursor(**kwargs)
@@ -82,7 +95,7 @@ class Database:
         cursor = self.cursor()
         
         if batch_size:
-            batches = std.batches(seq_params, batch_size)
+            [*batches] = std.batches(seq_params, batch_size)
             if verbose and len(batches) == 1:
                 verbose = False
                          
@@ -116,7 +129,7 @@ class Database:
         return [args for args in self.query("select column_name,data_type,nullable from all_tab_columns where owner='%s' and table_name='%s'" % (self.conn._con._kwargs['user'], table))]
 
     def desc_table(self, table):
-        return [*self.query("desc %s" % table)]
+        return [*self.query("desc " + format_table(table))]
 
     def __enter__(self):
         if self.conn is None:
@@ -125,7 +138,7 @@ class Database:
 
     def __exit__(self, *args):
         self.conn.close()
-        self.conn = None 
+        self.conn = None
 
 
 class MySQLConnector(Database):
@@ -138,7 +151,7 @@ class MySQLConnector(Database):
             desc = self.desc_table(table)
         except mysql.connector.errors.ProgrammingError as err:
             print(err)
-            m = re.search("1146 \(42S02\): Table '(\w+).(\w+)' doesn't exist", str(err))
+            m = re.search("1146 \(42S02\): Table '(\w+).(\S+)' doesn't exist", str(err))
             database, table = m.groups()
             print('database =', database)
             print('table =', table)
@@ -201,6 +214,7 @@ class MySQLConnector(Database):
                     return key
                 
             desc = {}
+            keys = [key for key in keys if dtype[key] is not None]
             for key in keys:
                 match dtype[key].__name__:
                     case 'bool':
@@ -261,7 +275,7 @@ class MySQLConnector(Database):
                 keys.insert(0, PRI)
                 desc[PRI] = 'int not null auto_increment'
             else:
-                desc[PRI] += ' not null' 
+                desc[PRI] += ' not null'
 
             def sort_key(key):
                 if key == PRI:
@@ -282,7 +296,7 @@ class MySQLConnector(Database):
             keys.sort(key=sort_key)
             print(keys)
 
-            desc = ',\n'.join(map(lambda key : f"`{key}` {desc[key]}", keys)) 
+            desc = ',\n'.join(map(lambda key : f"`{key}` {desc[key]}", keys))
             sql_create_table = f"CREATE TABLE `{database}`.`{table}` ({desc}, PRIMARY KEY (`{PRI}`)) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci PARTITION BY KEY() PARTITIONS 8"
             print(sql_create_table)
             try:
@@ -299,7 +313,7 @@ class MySQLConnector(Database):
         for i, (Field, Type, *_) in enumerate(desc):
             dtype[i] = Type
             
-            Type = str(Type, encoding="utf-8")  
+            Type = str(Type, encoding="utf-8")
             if Field == 'training':
                 has_training_field = True
                 
@@ -313,17 +327,22 @@ class MySQLConnector(Database):
             
             m = re.compile("varchar\((\d+)\)").match(Type)
             if m:
-                char_length[i] = int(m[1])           
+                char_length[i] = int(m[1])
                 
         truncate = kwargs.get('truncate')
         def create_csv(lines, step):
             import tempfile
             folder = tempfile.gettempdir()
                 
+            if '/' in table:
+                from std.file import mkdir
+                from os.path import dirname
+                mkdir(dirname(folder + '/' + table))
+
             for i in range(0, len(lines), step):
                 csv = folder + '/%s-%d.csv' % (table, i)
                 with open(csv, 'w', encoding='utf8') as file:
-                    for args in lines[i:i + step]: 
+                    for args in lines[i:i + step]:
                         if isinstance(args, tuple):
                             args = [*args]
                         elif isinstance(args, dict):
@@ -335,7 +354,7 @@ class MySQLConnector(Database):
 
                             if isinstance(arg, (list, tuple, dict)):
                                 cbytes = std.json_encode(std.json_encode(arg))[1:-1]
-                            elif isinstance(arg, str): 
+                            elif isinstance(arg, str):
                                 cbytes = std.json_encode(arg)[1:-1]
                             elif isinstance(arg, bytes):
                                 if len(arg) > char_length[i]:
@@ -403,11 +422,13 @@ class MySQLConnector(Database):
                 return self.load_data_from_csv(table, arg, **kwargs)
             return self.load_data_from_list(table, arg, **kwargs)
         else:
-            table_name = re.sub('\W', '_', table.lower())
-            [[count]] = self.query(f"select count(*) from information_schema.tables WHERE table_name = '{table_name}'")
-            if count:
-                print(table, 'already exists, skipping')
-                return
+            if replace := kwargs.get('replace'):
+                ...
+            else:
+                [[count]] = self.query(f"select count(*) from information_schema.tables WHERE table_name = '{table}'")
+                if count:
+                    print(table, 'already exists, skipping')
+                    return
             
             from datasets import load_dataset
             try:
@@ -416,7 +437,7 @@ class MySQLConnector(Database):
                 err = str(e)
                 print(err)
                 if m:= re.search("`load_dataset\('(\S+)', '(\S+)'\)`", err):
-                    assert m[1] == table
+                    assert table.endswith(m[1])
                     config = m[2]
                     data = load_dataset(table, config)
                 else:
@@ -433,14 +454,15 @@ class MySQLConnector(Database):
                     training = 2
                 else:
                     print('unrecognized key', key, 'from', data.keys())
-                    raise
+                    training = 3
                 
                 for obj in data[key]:
                     obj['training'] = training
                     array.append(obj)
-            return self.load_data(table_name, array)
+            return self.load_data(table, array, replace=replace)
 
     def load_data_from_csv(self, table, csv, delete=True, **kwargs):
+        table = format_table(table)
         start = time.time()
         csv = csv.replace('\\', '/')
         if replace := kwargs.get('replace'):
@@ -460,10 +482,10 @@ class MySQLConnector(Database):
         if not local_infile:
             self.execute('set global local_infile = 1')
             
-# in my.ini:            
+# in my.ini:
 # [mysql]
 # local-infile=1
-# 
+#
 # [mysqld]
 # local-infile=1
             
@@ -538,25 +560,35 @@ class MySQLConnector(Database):
         else:
             table = kwargs.pop('table')
 
-        sql = f"select {star} from {table} "
+        desc = self.desc_table(table)
+        desc = {Field: str(Type, encoding='utf8') for Field, Type, *_ in desc}
+
+        table = format_table(table)
+        sql = f"select %s from {table} "
         
         fetch_size = kwargs.pop('fetch_size', None)
         limit = kwargs.pop('limit', None)
-        if fetch_size:
-            assert limit, "limit must be set if fetch_size is set"
         
         where = kwargs.pop('where', None)
         offset = kwargs.pop('offset', None)
         order = kwargs.pop('order', None)
         dictionary = kwargs.pop('dictionary', None)
         conditions = []
+        
         for field, value in kwargs.items():
             if isinstance(value, (tuple, list, set)):
-                conditions.append(f"{field} in (%)" % ', '.join(str(t) for t in value))
+                value = ', '.join(std.json_encode(value) for t in value)
+                value = value.replace('%', '%%')
+                conditions.append(f"{field} in ({value})")
             else:
+                op = '='
                 if isinstance(value, str):
                     value = std.json_encode(value)
-                conditions.append(f"{field} = %s" % value)
+                    value = value.replace('%', '%%')
+                    if desc[field] in ('json', 'text'):
+                        op = 'regexp'
+
+                conditions.append(f"{field} {op} {value}")
           
         if where:
             conditions.append(where)
@@ -564,41 +596,52 @@ class MySQLConnector(Database):
         if conditions:
             sql += 'where ' + ' and '.join(conditions)
             
-        print('sql =', sql)
-        
-        if fetch_size and limit > fetch_size:
+        sql_select = sql % star
+        print('sql =', sql_select)
+        if fetch_size:
+            if limit is None:
+                [[limit]] = self.query(sql % 'count(*)')
+
             if offset is None:
                 offset = 0
+            
+            if order == 'rand()':
+                order = None
+                from random import shuffle
+                random.seed(1234)
+            else:
+                shuffle = lambda x : x
 
             for off in range(0, limit, fetch_size):
-                print('offset =', off + offset)
-                [*data] = self.query(sql, order=order, limit=fetch_size, offset=off + offset, dictionary=dictionary)
+                # print('offset =', off + offset)
+                [*data] = self.query(sql_select, order=order, limit=fetch_size, offset=off + offset, dictionary=dictionary)
+                shuffle(data)
                 yield from data
-                
         else:
-            yield from self.query(sql, order=order, limit=limit, offset=offset, dictionary=dictionary)
+            yield from self.query(sql_select, order=order, limit=limit, offset=offset, dictionary=dictionary)
 
     def __call__(self):
 #         import mysql.connector.pooling
-#          
+#
 #         config = {
 #           "user": "yourusername",
 #           "password": "yourpassword",
 #           "host": "yourhost",
 #           "database": "yourdatabase"
 #         }
-#          
+#
 #         cnxpool = mysql.connector.pooling.MySQLConnectionPool(pool_name="mypool", pool_size=20, **config)
-#          
+#
 #         cnx = cnxpool.get_connection()
-#          
+#
 #         cursor = cnx.cursor()
 #         cursor.execute("SELECT * FROM mytable")
-#          
+#
 #         cursor.close()
-#         cnx.close()        
+#         cnx.close()
         return MySQLConnector()
 
+
 instance = MySQLConnector()
 
 
@@ -625,13 +668,13 @@ def quote(s):
     return s.replace("'", "''").replace("\\", r"\\")
 
 
-def mysqlStr(Type, value):
-    if is_number(Type.decode()):
-        ... 
-    else:
-        value = std.json_encode(value)
-        
-    return value
+def mysqlStr(value, Type=None):
+    if isinstance(value, (int, float)):
+        return value
+    value = value.replace("'", "''")
+    value = std.json_encode(value)
+    return "'%s'" % value
+
 
 if __name__ == '__main__':
     ...
