@@ -22,7 +22,7 @@ def HeapPermutation(k, A):
 
 
 def generate_all_permutation(A):
-    yield from HeapPermutation(len(A), A)        
+    yield from HeapPermutation(len(A), A)
 
         
 def skip_first_permutation(A):
@@ -55,7 +55,7 @@ class TopKHeap:
     def topk(self):
         
         arr = []
-        while self.data: 
+        while self.data:
             a = heapq.heappop(self.data)
             arr.append(a)
             
@@ -71,7 +71,7 @@ def generate_combination(n, k):
     while True:
         if x[i] <= n - (k - i):
             if i == k - 1:
-                yield [*x]                
+                yield [*x]
             else:
                 i += 1
                 x[i] = x[i - 1]
@@ -82,13 +82,13 @@ def generate_combination(n, k):
         if x[0] > n - k:
             break
 
-from _functools import reduce
+from functools import reduce
 
 def sigmar(x, k):
     if k == 0:
         return 1
     if k == 1:
-        return sum(x) 
+        return sum(x)
     sigmar = 0
     for indices in generate_combination(len(x), k):
         sigmar += reduce(lambda x, y: x * y, (x[i] for i in indices))
@@ -100,3 +100,7 @@ def random_combination(n, k, reverse=True):
     indices = [all_indices.pop(random.randrange(0, len(all_indices))) for _ in range(k)]
     indices.sort(reverse=reverse)
     return indices
+
+if __name__ == '__main__':
+    for a in generate_all_permutation([*range(4)]):
+        print(a)
