@@ -1,47 +1,58 @@
 import numpy as np
-import random
+import random, os
 
-def extend(arr, mask, maxlength, padding_type):
+
+def extend(arr, mask, maxlength, padding_side):
     if isinstance(arr, (tuple, np.ndarray)):
         arr = [*arr]
 
     padding = [mask] * (maxlength - len(arr))
-    if padding_type == 'tailing':
+    if padding_side == 'right':
         arr.extend(padding)
-    elif padding_type == 'leading':
+    elif padding_side == 'left':
         arr = padding + arr
     else:
-        assert padding_type == 'general'
         for mask in padding:
             arr.insert(random.randrange(0, len(arr)), mask)
 
     return arr
 
 
-def extend_tailing(arr, mask, maxlength):
+def extend_right(arr, mask, max_length):
     if isinstance(arr, tuple):
         arr = [*arr]
 
-    padding = [mask] * (maxlength - len(arr))
+    padding = [mask] * (max_length - len(arr))
 
     arr.extend(padding)
 
     return arr
 
 
-def numpify_tailing(arr, mask_value=0):
+def padding_right(arr, mask_value=0):
     maxWidth = max(len(x) for x in arr)
     # arr is a 2-dimension array
     for i in range(len(arr)):
-        arr[i] = extend_tailing(arr[i], mask_value, maxWidth)
+        arr[i] = extend_right(arr[i], mask_value, maxWidth)
     return np.array(arr)
 
 
-def numpify_leading(arr, mask_value=0):
-    return numpify(arr, mask_value, padding='leading')
+def padding_left(arr, mask_value=0):
+    return padding(arr, mask_value, padding_side='left')
 
 
-def numpify(arr, mask_value=0, padding='tailing', dtype=None):
+def get_padding_left(input_ids):
+    try:
+        i = 0
+        while True:
+            if any(input[i] for input in input_ids):
+                return i
+            i += 1
+    except IndexError:
+        raise Exception('input with the minimum length consists of all zeros!')
+
+
+def padding(arr, mask_value=0, padding_side='right', dtype=None):
     '''
     
     :param arr:
@@ -52,28 +63,26 @@ def numpify(arr, mask_value=0, padding='tailing', dtype=None):
     try:
         maxWidth = max(len(x) for x in arr)
     except (TypeError, AttributeError) as _:
-        return np.array(arr)
+        return np.array(arr, dtype=dtype)
 
     try:
         maxHeight = max(max(len(word) for word in x) for x in arr)
         for i in range(len(arr)):
             for j in range(len(arr[i])):
-                arr[i][j] = extend(arr[i][j], mask_value, maxHeight, padding)
-            arr[i] = extend(arr[i], [mask_value] * maxHeight, maxWidth, padding)
+                arr[i][j] = extend(arr[i][j], mask_value, maxHeight, padding_side)
+            arr[i] = extend(arr[i], [mask_value] * maxHeight, maxWidth, padding_side)
     except (TypeError, AttributeError, ValueError) as _:
 
         # arr is a 2-dimension array
         try:
             for i in range(len(arr)):
-                arr[i] = extend(arr[i], mask_value, maxWidth, padding)
+                arr[i] = extend(arr[i], mask_value, maxWidth, padding_side)
         except AttributeError as _:
             #arr might be an array of string
             ...
 
-    arr = np.array(arr)
-    if dtype:
-        arr = arr.astype(dtype)
-    return arr
+    return np.array(arr, dtype=dtype)
+
 
 def randomize(data, count):
     from std.combinatorics import random_combination
@@ -83,7 +92,8 @@ def randomize(data, count):
         data[i], data[j] = data[j], data[i]
 
     return data
-    
+
+
 def sample(data, count):
     '''
     this sampling ensure dislocation arrangement
@@ -104,6 +114,25 @@ def sample(data, count):
     else:
         return sample(data * quotient, count)
 
+
+def create_memmap(filename, mode='r'):
+    dtype = np.dtype(filename[filename.rindex('.') + 1:])
+    return np.memmap(
+        filename,
+        dtype=dtype,
+        mode=mode,
+        shape=(os.path.getsize(filename) // dtype.alignment,))
+
+
+def get_len_padded(input):
+    for i, id in enumerate(input):
+        if id:
+            return i
+
+def get_seq_length(input):
+    return len(input) - get_len_padded(reversed(input))
+
+
 if __name__ == '__main__':
     count = 22
     data = [*range(10)]
