@@ -2,7 +2,7 @@ import os
 import json
 
 def createNewFile(path):
-    createNewPath(os.path.dirname(path))
+    mkdir(os.path.dirname(path))
 
     try:
         os.mknod(path)
@@ -11,9 +11,8 @@ def createNewFile(path):
             ...
 
 
-def createNewPath(basedir):
-    if not os.path.exists(basedir):
-        os.makedirs(basedir)
+def mkdir(basedir):
+    os.makedirs(basedir, exist_ok=True)
 
 
 class Text:
@@ -237,7 +236,7 @@ class Text:
         tell = self.tell()
         self.end()
         size = self.file.tell()
-        self.file.seek(tell, os.SEEK_SET)        
+        self.file.seek(tell, os.SEEK_SET)
         return size
         
     def endswith(self, end):
@@ -249,7 +248,7 @@ class Text:
         
         _offset = self.file.seek(offset, os.SEEK_SET)
         assert _offset == offset
-        char = self.file.read(size)        
+        char = self.file.read(size)
         return char == end
 
     def readlines(self):
@@ -257,12 +256,12 @@ class Text:
         return [s for s in self]
     
     def readline(self):
-        try: 
+        try:
             return next(self)
         except StopIteration:
             return ''
     
-    def writelines(self, array): 
+    def writelines(self, array):
         self.rewind()
 
         for s in array:
@@ -289,7 +288,7 @@ class GzipFile:
         import gzip
         try:
             self.file = gzip.open(path, "rb" if os.path.isfile(path) else "wb")
-        except PermissionError: 
+        except PermissionError:
             self.file = gzip.open(path, "rb")
         except FileNotFoundError:
             createNewFile(path)
@@ -320,14 +319,14 @@ class GzipFile:
         self.file.write(s.encode())
         self.file.write(b'\n')
 
-    def readlines(self): 
+    def readlines(self):
         return self.file.readlines()
     
     def readline(self):
-        try: 
+        try:
             return next(self)
         except StopIteration:
-            return ''    
+            return ''
 
 
 def read_excel(file):
