@@ -1,8 +1,20 @@
 # -*- coding: UTF-8 -*-
 import regex as re
 
-def detect_language(text):
-    if re.compile('\p{Han}{2,}').match(text):
+Han1 = re.compile('\p{Han}')
+Han2 = re.compile('\p{Han}{2,}')
+
+def detect_language(text, fast=True):
+    if fast:
+        if len(text) <= 2:
+            if Han1.search(text):
+                return 'cn'
+        else:
+            if Han2.search(text):
+                return 'cn'
+        return 'en'
+
+    if Han2.match(text):
         return 'cn'
 
     cn = 0
@@ -13,7 +25,7 @@ def detect_language(text):
         position_weight = (i - mid + 0.01) / mid
         position_weight = position_weight ** 2
         if re.compile('\p{Letter}').match(ch):
-            if re.compile('\p{Han}').match(ch):
+            if Han1.match(ch):
                 cn += 8 * position_weight
             else:
                 en += position_weight
