@@ -12,62 +12,9 @@ instance.ahocorasick_jws_split_s[VectorS] = MatrixS
 instance.ahocorasick_jws_split_s_digits[VectorS] = MatrixS
 # instance.ahocorasick_jws_split_s_debug[VectorS] = MatrixS
 
-
-def tagCorpus():
-    from std import MySQL
-    from model.syntax import compiler
-    import MeCab
-    posTagger = MeCab.Tagger()
-    
-    lines = []
-    
-    corpus = {}
-    for infix, *_ in MySQL.instance.select('select text from tbl_claim_parser_jp limit 5000 offset 15000'):
-        
-        tree = compiler.TreeNode.compile(infix)
-        assert str(tree) == infix
-        
-        assert isinstance(tree, compiler.TreeNodeArray)
-        
-        _, *claims = tree.array
-        claims = [compiler.original_text(claim) for claim in claims]
-
-        for text in claims:
-            for line in sbd(text):
-                if len(line) < 32:
-                    continue
-                
-                arr = [*split_sentence_limited(line, 128)]
-                
-                for text in arr:
-                    result = posTagger.parse(text).split('\n')
-                    *words, EOS, _ = (lexeme.split('\t')[0] for lexeme in result)                    
-                    
-#                     print(words)
-                    text_ = convertToOriginal(words)                    
-                    if text_ != text:
-                        corpus[text_] = convertFromSegmentation(words)
-                    else:
-                        corpus[text] = convertFromSegmentation(words)
-                        
-                    
-                assert ''.join(arr) == line, 'result is not the same as original text!'
-                lines.extend(arr)
-                
-        
-    print('len(corpus) =', len(corpus))
-    
-    data = []
-    for text, segment in corpus.items():
-        data.append((text, segment, 1))
-        
-    MySQL.instance.load_data('tbl_segment_jp', data)
-#             preamble = self.split(preamble)
-#             claims = self.split(claims)
-        
     
 @singledispatch
-def normalize(text): 
+def normalize(text):
     return normalize_n(text)
 
 
@@ -77,15 +24,15 @@ def _(text):
 
 
 def normalize_n(word):
-    word = re.compile('\n').sub(r'\\n', word)    
+    word = re.compile('\n').sub(r'\\n', word)
     word = word.strip()
     word = word.lower()
     return word
 
 
 def normalize_d(word):
-    word = re.compile('\n').sub(r'\\n', word)    
-    word = re.compile('\d').sub(r'\d', word)    
+    word = re.compile('\n').sub(r'\\n', word)
+    word = re.compile('\d').sub(r'\d', word)
     word = word.strip()
     word = word.lower()
     return word
@@ -135,7 +82,7 @@ def convertFromSegmentation(arr):
     return s
 
 
-def convertToSegmentation(s):    
+def convertToSegmentation(s):
     s = re.compile("([" + sPunctuation + "])").sub(' \\1 ', s)
 
     s = re.compile('(?<=[\\d])( +([\\.．：:]) +)(?=[\\d]+)').sub('\\2', s)
@@ -148,9 +95,9 @@ def convertToSegmentation(s):
     return s.strip().split()
 
 
-def isEnglish(ch):    
+def isEnglish(ch):
     return 'a' <= ch <= 'z' or 'A' <= ch <= 'Z' or 'ａ' <= ch <= 'ｚ' or 'Ａ' <= ch <= 'Ｚ'\
-        or '0' <= ch <= '9' or '０' <= ch <= '９'        
+        or '0' <= ch <= '9' or '０' <= ch <= '９'
 #     return re.compile('[a-zA-Zａ-ｚＡ-Ｚ\d]').match(ch)
 
 
@@ -177,11 +124,11 @@ def establish_vocabFile():
             seg = convertToSegmentation(segement)
             _text = convertToOriginal(seg)
             if text != _text:
-#                 print(text) 
+#                 print(text)
 #                 print(_text)
 #                 print(' '.join(seg))
                 deletes.append((text,))
-                updates.append((_text, segement)) 
+                updates.append((_text, segement))
             for seg in seg:
                 if len(seg) == 1:
                     continue
@@ -191,7 +138,7 @@ def establish_vocabFile():
                 else:
                     dic[seg] = 1
             
-        for key in dic:        
+        for key in dic:
             dic[key] = math.sqrt(dic[key] * len(key))
             
         utility.normalize(dic)
