@@ -23,7 +23,7 @@ class XMLNode:
     def root(self):
         while self.parent:
             self = self.parent
-        return self 
+        return self
 
     def has_recursive_error(self):
         try:
@@ -220,7 +220,7 @@ class XMLNodeText(XMLNode):
         return self.arg.text
     
     @computed
-    def physicalText(self):        
+    def physicalText(self):
         return str(self.arg)
     
     @property
@@ -241,7 +241,7 @@ class XMLNodeText(XMLNode):
         return 0 <= pos < self.length
 
     def is_sibling(self, start, end):
-        return 0 <= start < end <= self.length 
+        return 0 <= start < end <= self.length
         
     def getPhysicalIndices(self, start, stop):
         return start, stop
@@ -343,7 +343,7 @@ class XMLNodeBinaryTag(XMLNode):
 
     @property
     def is_unbalanced(self):
-        return not self.tagEnd or isinstance(self.tagEnd, XMLNodeUnbalancedTag) 
+        return not self.tagEnd or isinstance(self.tagEnd, XMLNodeUnbalancedTag)
     
     def logical2physical(self, pos):
         return self.arg.logical2physical(pos) + self.tagBegin.length
@@ -389,7 +389,7 @@ class XMLNodeBinaryTag(XMLNode):
         else:
             node.parent = self
 
-    def enumerate(self):        
+    def enumerate(self):
         yield from self.ptr.enumerate()
 
     def has(self, cls):
@@ -542,7 +542,7 @@ class XMLNodeSingleTag(XMLNode):
     def getPhysicalIndices(self, start, stop):
         return 0, self.arg.length
 
-    def enumerate(self):        
+    def enumerate(self):
         yield from self.ptr.enumerate()
 
     def has(self, cls):
@@ -654,7 +654,7 @@ class XMLNodeArray(XMLNode):
         else:
             node.parent = self
 
-    def enumerate(self):        
+    def enumerate(self):
         for node in self.args:
             yield from node.enumerate()
 
@@ -774,11 +774,11 @@ class XMLNodeArray(XMLNode):
         _index = binary_search(logicalOffset, stop - 1, self.cmp)
         
         if index == _index:
-            prev_start = logicalOffset[index][0]    
+            prev_start = logicalOffset[index][0]
             return add(self.args[index].getPhysicalIndices(start - prev_start, stop - prev_start), prev_start + offsets[index])
         else:
             [prev_start, prev_stop] = logicalOffset[index]
-            _prev_start = logicalOffset[_index][0]    
+            _prev_start = logicalOffset[_index][0]
             start = add(self.args[index].getPhysicalIndices(start - prev_start, prev_stop - prev_start), prev_start + offsets[index])[0]
             stop = add(self.args[_index].getPhysicalIndices(0, stop - _prev_start), _prev_start + offsets[_index])[1]
             return [start, stop]
@@ -838,65 +838,10 @@ class XMLNodeUnbalancedTag(XMLNode):
     def zeros(self):
         return [0]
     
-        
-def compile(infix):
-    caret = XMLNodeCaret()
-
-    leftTagCount = {}
-    for text in infix:
-        if text.is_TagBegin:
-            caret = caret.append_left_tag(text)
-            if text.tag not in leftTagCount:
-                leftTagCount[text.tag] = []
-            leftTagCount[text.tag].append(caret.parent)
-#            assert caret.parent.tagBegin is text, "caret.parent.tagBegin is text"
-            
-        elif text.is_TagEnd:
-            old = caret
-            while True:
-                if caret:
-                    root = caret
-                    caret = caret.parent
-                    if isinstance(caret, XMLNodeBinaryTag) and caret.tag == text.tag:
-#                         assert caret.tagEnd is None
-                        caret.set_tagEnd(text)
-                        while True:
-                            _caret = leftTagCount[text.tag].pop()
-                            if caret is _caret:
-                                break
-                            _caret.reduceToNodeText()
-#                         index = leftTagCount[text.tag].index(caret)
-#                         leftTagCount[text.tag].pop(index)
-                        break
-                elif leftTagCount[text.tag]:
-                    parent = leftTagCount[text.tag].pop()
-                    caret = XMLNodeUnbalancedTag(text, parent)
-                    while parent.stop < root.stop:
-                        parent = parent.parent
-                        
-                    caret = parent.append_tag(caret)
-                    break
-                else:
-                    text = text.reduceToNodeText()
-                    caret = old.append_text_node(text)
-                    break
-                
-        elif text.is_TagSingle or text.is_HTMLEntity:
-            caret = caret.append_single_tag(text)
-        else:
-#             assert text.is_PlainText
-            caret = caret.append_text_node(text)
-        
-    for tag in leftTagCount:
-        while leftTagCount[tag]:
-            leftTagCount[tag].pop().reduceToNodeText()
-
-    return caret.root
-    
     
 def style_type(ptr, style):
     offsetStart = ptr.offsetStart
-    offsetStop = ptr.offsetStop        
+    offsetStop = ptr.offsetStop
     this_set = Range(offsetStart, offsetStop)
     
     style_intersected = {}
@@ -973,17 +918,17 @@ def detect_style(interval, style):
     return interval
 
 tagName = '([a-z][-:_a-z]*\d*)'
-attrList = '''(?:\s+:?[a-z][-:_a-z]*=(?:"[^"]*"|'[^']*'))*\s*'''    
+attrList = '''(?:\s+:?[a-z][-:_a-z]*=(?:"[^"]*"|'[^']*'))*\s*'''
 tagBegin = f'<{tagName}{attrList}>'
 tagRegex = fr'''{tagBegin}(?=[\s\S]*?<\/\1>)|</{tagName}>|<(img|mspace|br){attrList}/>|&(#[0-9]+|#x[0-9a-f]+|[^\t\n\f <&#;]{{1,32}});'''
 
 # print("tagRegex =", tagRegex)
 
-def construct_rich_text(text):
+def compile(text):
     start = 0
     
     richTexts = []
-    leftTagCount = {} 
+    leftTagCount = {}
     for m in re.finditer(tagRegex, text, re.IGNORECASE):
 #                             1-----------------)                                                                     2-----------------)   3-------------)                                                   4----------------------------------------)
         prevText = text[start:m.start()]
@@ -1019,7 +964,58 @@ def construct_rich_text(text):
     if restText:
         richTexts.append(PlainText(text, start, len(text)))
 
-    return compile(richTexts)
+    caret = XMLNodeCaret()
+
+    leftTagCount = {}
+    for text in richTexts:
+        if text.is_TagBegin:
+            caret = caret.append_left_tag(text)
+            if text.tag not in leftTagCount:
+                leftTagCount[text.tag] = []
+            leftTagCount[text.tag].append(caret.parent)
+#            assert caret.parent.tagBegin is text, "caret.parent.tagBegin is text"
+            
+        elif text.is_TagEnd:
+            old = caret
+            while True:
+                if caret:
+                    root = caret
+                    caret = caret.parent
+                    if isinstance(caret, XMLNodeBinaryTag) and caret.tag == text.tag:
+#                         assert caret.tagEnd is None
+                        caret.set_tagEnd(text)
+                        while True:
+                            _caret = leftTagCount[text.tag].pop()
+                            if caret is _caret:
+                                break
+                            _caret.reduceToNodeText()
+#                         index = leftTagCount[text.tag].index(caret)
+#                         leftTagCount[text.tag].pop(index)
+                        break
+                elif leftTagCount[text.tag]:
+                    parent = leftTagCount[text.tag].pop()
+                    caret = XMLNodeUnbalancedTag(text, parent)
+                    while parent.stop < root.stop:
+                        parent = parent.parent
+                        
+                    caret = parent.append_tag(caret)
+                    break
+                else:
+                    text = text.reduceToNodeText()
+                    caret = old.append_text_node(text)
+                    break
+                
+        elif text.is_TagSingle or text.is_HTMLEntity:
+            caret = caret.append_single_tag(text)
+        else:
+#             assert text.is_PlainText
+            caret = caret.append_text_node(text)
+        
+    for tag in leftTagCount:
+        while leftTagCount[tag]:
+            leftTagCount[tag].pop().reduceToNodeText()
+
+    return caret.root
 
 
 class XMLText:
@@ -1117,7 +1113,7 @@ def cmp_range(a, b):
     return 0
     
     
-def is_alphanum(ch):    
+def is_alphanum(ch):
     return re.compile('[\p{Letter}\d]').match(ch)
 
 
@@ -1156,9 +1152,10 @@ def convertToOriginalWithOffsets(arr):
     
     return s, offsets
 
+
 def test():
     physicalText = "firstly,&nbsp;<div><b>this&nbsp;is&nbsp;<I>an&nbsp;<font color='red'><i>italic</i>&nbsp;<u>&lt;underline&gt;</u><mspace /></font>, simply put</b>,&nbsp;this&nbsp;is&nbsp;an&nbsp;italic&nbsp;text,&nbsp;</I>with&nbsp;something&emsp;<u>&lt;underlined&gt;</u>;</div>&ensp;<b>another&nbsp;bold&nbsp;text&nbsp;is&nbsp;followed.&nbsp;</b>At&nbsp;last,&nbsp;there&nbsp;a&nbsp;plain&nbsp;text."
-    richText = construct_rich_text(physicalText)
+    richText = compile(physicalText)
     # print(richText)
     richText.sanctity_check()
     print("physical text =", richText.physicalText)
@@ -1181,6 +1178,7 @@ def test():
     print("its original physical text is:")
     print(physicalText[start:stop])
     print()
-    
+
+
 if __name__ == '__main__':
     test()
\ No newline at end of file
